## tpad version 1.3
# WinXP (TM) enhanced Notepad clone written in Tcl/Tk.
# HTML TIDY plugin
# Copyright (C) 2004  Antonio Bonifati
#
# Please see the main file tpad.tcl for licence and disclaimer
#
## last revised on: $Date: 2004/08/31 $

package provide tpad 1.3

package require fileutil 1.6  ;# uses temporary files

# make sure the msgcat namespace exists
namespace eval ::msgcat {}

namespace eval ::tpad::tidy {
  namespace export display

  namespace import ::msgcat::mc ::msgcat::mcmax
}

proc ::tpad::tidy::DisplaySettings {pn} {
  variable ::tpad::$pn\::p
  variable ::tpad::$pn\::edit
  foreach v {config column} {
    variable $pn\::$v
  }

  set m $p.tidy

  set w $m.f.conf.file
  $w delete 0 end
  $w insert 0 $config

  if {$edit} {
    set w $m.f.wrap.column
    $w configure -state normal
    $w delete 0 end
    $w insert 0 $column

    EnableEntry $pn
  }

  EnableCheckbuttons $pn
  EnableButton $pn $config
}

proc ::tpad::tidy::HideShowOptions {pn} {
  variable ::tpad::$pn\::p
  variable $pn\::showopts

  set m $p.tidy
  if {$showopts} {
    $m.s configure -text [mc "Hide options"]

    grid $m.f
    grid $m.f2
  } else {
    $m.s configure -text [mc "Show options"]

    grid remove $m.f
    grid remove $m.f2
  }
}

proc ::tpad::tidy::ToggleOptions {pn} {
  variable ::tpad::$pn\::p
  variable $pn\::showopts

  set showopts [expr !$showopts]
  HideShowOptions $pn
}

proc ::tpad::tidy::SetConfig {pn} {
  variable ::tpad::$pn\::p
  variable $pn\::config

  set m $p.tidy

  set newFilename [tk_getOpenFile -title [mc "Select - tpad"] -parent $m \
    -filetypes $::tpad::types]

  if {$newFilename!={}} {
    set w $m.f.conf.file
    $w delete 0 end
    $w insert 0 $newFilename
  }
}

proc ::tpad::tidy::EditConfig {pn} {
  variable ::tpad::$pn\::p

  set m $p.tidy

  toplevel $m.conf
  grab $m.conf
  ::tpad::tpad $m.conf tpad [$m.f.conf.file get]
  ::tpad::display $m.conf
}

proc ::tpad::tidy::GoToError {pn} {
  variable ::tpad::$pn\::p

  set w $p.tidy.log.err

  set selIdx [$w curselection]
  set line [$w get $selIdx]

  if { [scan $line {%d:%d:} row col] == 2 } {
    ::tpad::goToLine $pn $row [expr $col - 1]
  }
}

proc ::tpad::tidy::WriteSettings {pn} {
  variable ::tpad::$pn\::p
  variable ::tpad::$pn\::edit
  foreach v {config wrap column} {
    variable $pn\::$v
  }

  set m $p.tidy

  set w $m.f.wrap
  if {$edit} {
    if {$wrap && [$w.column get]=={}} {
      tk_messageBox -title [mc "HTML Tidy"] -icon error -parent $m \
        -message [mc "wrapping column can't be empty"]
      ::tk::TabToWindow $w.column
      return no
    }

    set column [$w.column get]
  }

  set w $m.f.conf.file
  set config [file normalize [$w get]]
  if {$config!={} && (![file isfile $config] || ![file readable $config])} {
      tk_messageBox -title [mc "HTML Tidy"] -icon error -parent $m \
        -message [mc "configuration file must exist and be readable"]
      ::tk::TabToWindow $w
      return no
  }
  return yes
}

proc ::tpad::tidy::InitSettings {pn} {
  global env
  namespace eval $pn {}

  foreach var {showopts config indent wrap column preserve \
      xml omit upper clean asxml numeric} {
    set value [set ::tpad::tidy::$var]

    variable $pn\::$var $value
  }

  if {$config=={} && [info exists env(HTML_TIDY)]} {
    set config $env(HTML_TIDY)
  }
}

# modify (no, yes): modify text in place (update original text)
proc ::tpad::tidy::ProcessText {pn modify} {
  variable ::tpad::$pn\::p
  variable ::tpad::$pn\::filename
  foreach v {config indent wrap column preserve \
      xml omit upper clean asxml numeric} {
    variable $pn\::$v
  }

  # generate the most simple command

  set cmd [list $::tpad::tidy]
  set errFile [::fileutil::tempfile tpad.]
  set outFile [::fileutil::tempfile tpad.]

  set opts q
  if {$modify} {
    if {$indent} {append opts i}
    if {!$xml} {
      if {$omit} {append opts o}
      if {$upper} {append opts u}
      if {$clean} {append opts c}
      if {!$preserve && $numeric} {append opts n}
    }
    append opts m
  } else {
    append opts e
  }
  lappend cmd -$opts

  if {$config!={}} {
    lappend cmd -config $config
  }

  if {$modify} {
    if {$wrap} {
      if {$column!=68} {lappend cmd -wrap $column}
    } else {
      lappend cmd -wrap 0
    }

    if {$preserve} {lappend cmd -preserve}
  }

  if {$xml} {
    lappend cmd -xml
  } elseif {$modify} {
    if {$asxml} {lappend cmd -asxml}
  }

  lappend cmd --gnu-emacs yes

  lappend cmd -f $errFile

  # execute it and capture output

  if {$modify} {
    catch {eval exec $cmd >$outFile << {[$p.text get 1.0 end]}}
  } else {
    catch {eval exec $cmd << {[$p.text get 1.0 end]}}
  }

  # parse error output

  set w $p.tidy.log.err
  $w delete 0 end
  set fileid [open $errFile]
  while {[gets $fileid line]!=-1} {
    # parse this line
    if { [scan $line {(null):%d:%d: %s %n} row col type msgIdx] == 0} {

      # an advice spans multiple lines and is terminated by an empty line
      while {yes} {
        set msg $line
        while { [set line [gets $fileid]]!={} } {
          append msg " $line"
        }
        $w insert end $msg

        if {[gets $fileid line]==-1} break
      }

      break
    }

    # an error message always takes a single line
    set msg [string range $line $msgIdx end]
    $w insert end "$row:$col: $type $msg"
    switch $type {
      Error: {
        $w itemconfigure end -foreground $::tpad::tidy::errorFg \
          -background $::tpad::tidy::errorBg
      }
      Warning: {
        $w itemconfigure end -foreground $::tpad::tidy::warningFg \
          -background $::tpad::tidy::warningBg
      }
    }
  }
  close $fileid

  if {$modify} {
    # replace text with the pretty printed version,
    # if Tidy was able to generated it

    set fileid [open $outFile]
    set pretty [read $fileid]
    close $fileid

    if {$pretty!={}} {
      bind $w <<ListboxSelect>> {}

      set w $p.text

      $w configure -autosep no
      $w edit separator

      $w delete 1.0 end
      $w insert 1.0 $pretty

      $w edit separator
      $w configure -autosep yes

      $w mark set insert 1.0
    } else {
      bind $w <<ListboxSelect>> [namespace code "GoToError $pn"]
    }
  } else {
    bind $w <<ListboxSelect>> [namespace code "GoToError $pn"]
  }

  file delete $errFile $outFile
}

proc ::tpad::tidy::EnableEntry {pn} {
  variable ::tpad::$pn\::p
  variable ::tpad::$pn\::edit
  variable $pn\::wrap

  if {$edit} {
    set m $p.tidy
    set state [expr {$wrap ? {normal} : {disabled}}]
    foreach wid {label column} {
      $m.f.wrap.$wid configure -state $state
    }
  }
}

proc ::tpad::tidy::EnableButton {pn newval} {
  variable ::tpad::$pn\::p

  set w $p.tidy.f.conf.ed
  if {$newval=={}} {
    $w configure -state disabled
  } else {
    $w configure -state normal
  }
  return yes
}

proc ::tpad::tidy::EnableCheckbuttons {pn} {
  variable ::tpad::$pn\::p
  variable ::tpad::$pn\::edit
  variable $pn\::xml

  if {$edit} {
    set m $p.tidy
    set state [expr {$xml ? {disabled} : {normal}}]

    foreach wid {o u c asxml} {
      $m.f2.htm.$wid configure -state $state
    }

    EnableCheckbutton $pn
  }
}

proc ::tpad::tidy::EnableCheckbutton {pn} {
  variable ::tpad::$pn\::p
  variable ::tpad::$pn\::edit
  variable $pn\::preserve
  variable $pn\::xml

  if {$edit} {
    set m $p.tidy
    set state [expr {$xml || $preserve ? {disabled} : {normal}}]

    $m.f2.htm.n configure -state $state
  }
}

proc ::tpad::tidy::display {pn} {
  variable ::tpad::$pn\::p
  variable ::tpad::$pn\::edit

  if {![namespace exists $pn]} {
    InitSettings $pn
  }

  set m $p.tidy
  toplevel $m
  wm group $m $pn
  wm title $m [mc "HTML Tidy"]
  wm transient $m $pn
  wm resizable $m yes yes
  # catch the kill of the window manager
  wm protocol $m WM_DELETE_WINDOW [namespace code "
    if {\[WriteSettings $pn\]} {
      set ::tpad::$pn\::ht no
      destroy $m
    }
  "]

  ::tpad::AmpWidget button $m.s -font $::tpad::tidy::smallFont \
    -comm [namespace code "ToggleOptions $pn"]

  frame $m.f -padx 1.5m -pady 1.5m

  if {$edit} {
    ::tpad::AmpWidget checkbutton $m.f.i -var ::tpad::tidy::$pn\::indent \
      -text [mc "&indent element content"] -onv yes -offv no

    set w $m.f.wrap
    frame $w
    ::tpad::AmpWidget checkbutton $w.w -text [mc "&wrap text"] \
      -var ::tpad::tidy::$pn\::wrap -comm [namespace code "EnableEntry $pn"] \
      -onv yes -offv no
    label $w.label -text [mc "at column"]
    spinbox $w.column -from 1 -to 2000 -width 6 -validate key \
      -vcmd [namespace code {string is integer %P}]
    pack $w.w $w.label $w.column -side left

    ::tpad::AmpWidget checkbutton $m.f.preserve \
      -var ::tpad::tidy::$pn\::preserve -onv yes -offv no \
      -text [mc "&preserve entities from source text"] \
      -comm [namespace code "EnableCheckbutton $pn"]

    pack $m.f.i $m.f.wrap $m.f.preserve -side top -anchor w
  }

  set w $m.f.conf
  labelframe $w -text [mc "configuration file:"] -padx 1.5m -pady 1.5m
  entry $w.file -validate all -vcmd [namespace code "EnableButton $pn %P"]
  ::tpad::AmpWidget button $w.sel -text [mc "&Select"] \
    -comm [namespace code "SetConfig $pn"]
  ::tpad::AmpWidget button $w.ed -text [mc "&Edit"] \
    -comm [namespace code "EditConfig $pn"]
  pack $w.file -side top -fill x
  pack $w.sel $w.ed -side left -pady 1.5m -padx {0 1.5m}

  pack $m.f.conf -side top -anchor w -fill x

  frame $m.f2 -padx 1.5m -pady 1.5m

  ::tpad::AmpWidget radiobutton $m.f2.xml -var ::tpad::tidy::$pn\::xml \
    -value yes -text [mc "input is wellformed &XML"] \
    -comm [namespace code "EnableCheckbuttons $pn"]

  if {$edit} {
    set w $m.f2.htm
    labelframe $w -padx 1.5m -pady 1.5m

    ::tpad::AmpWidget radiobutton $w.html -var ::tpad::tidy::$pn\::xml \
      -value no -text [mc "input is &HTML"] \
      -comm [namespace code "EnableCheckbuttons $pn"]
    $m.f2.htm configure -labelwidget $w.html

    ::tpad::AmpWidget checkbutton $w.o -var ::tpad::tidy::$pn\::omit \
      -text [mc "omit optional en&d-tags"] -onv yes -offv no
    ::tpad::AmpWidget checkbutton $w.u -var ::tpad::tidy::$pn\::upper \
      -text [mc "force tags in &upper case"] -onv yes -offv no
    ::tpad::AmpWidget checkbutton $w.c -var ::tpad::tidy::$pn\::clean \
      -text [mc "clean using &CSS"] -onv yes -offv no
    ::tpad::AmpWidget checkbutton $w.asxml -onv yes -offv no \
      -var ::tpad::tidy::$pn\::asxml \
      -text [mc "conver&t HTML to wellformed XML"]
    ::tpad::AmpWidget checkbutton $w.n -var ::tpad::tidy::$pn\::numeric \
      -text [mc "output e&ntities as XML"] -onv yes -offv no
    pack $w.o $w.u $w.c $w.asxml $w.n -side top -anchor w

    pack $m.f2.xml -side top -anchor w -padx {2m 0}
    pack $m.f2.htm -side top -anchor w -fill x
  } else {
    ::tpad::AmpWidget radiobutton $m.f2.htm -var ::tpad::tidy::$pn\::xml \
      -value no -text [mc "input is &HTML"]
    pack $m.f2.xml $m.f2.htm -side top -anchor w
  }

  set w $m.but
  frame $w
  ::tpad::AmpWidget button $w.validate -text [mc &Validate] -comm [namespace code "
    if {\[WriteSettings $pn\]} {
      ProcessText $pn no
    }
  "]
  ::tpad::AmpWidget button $w.restore -text [mc "&Restore options"] -comm [
    namespace code "InitSettings $pn; DisplaySettings $pn"
  ]
  ::tpad::AmpWidget button $w.close -text [mc Cl&ose] -comm [
  namespace code "
    if {\[WriteSettings $pn\]} {
      set ::tpad::$pn\::ht no
      focus $p.text
      after idle {destroy $m}
    }
  "]
  if {$edit} {
    ::tpad::AmpWidget button $w.correct -text [mc "V&alidate and correct"] \
        -comm [namespace code "
      if {\[WriteSettings $pn\]} {
        ProcessText $pn yes
      }
    "]
    pack $w.validate $w.correct $w.restore $w.close -side left -exp yes
  } else {
    pack $w.validate $w.restore $w.close -side left -exp yes
  }

  set w $m.log
  frame $w
  bind [::tpad::AmpWidget label $w.lab \
    -text [mc "&Last Error Messages and Advice"]] \
    <<AltUnderlined>> [list ::tk::TabToWindow $w.err]
  listbox $w.err -setgrid yes \
    -yscroll "$w.vscroll set" -xscroll "$w.hscroll set"
  scrollbar $w.vscroll -comm "$w.err yview"
  scrollbar $w.hscroll -orient horizontal -comm "$w.err xview"
  grid $w.lab -row 0 -column 0 -columnspan 2
  grid $w.err -row 1 -column 0 -sticky news
  grid $w.vscroll -row 1 -column 1 -sticky ns
  grid $w.hscroll -row 2 -column 0 -sticky ew
  grid rowconfigure $w 1 -weight 1
  grid columnconfigure $w 0 -weight 1

  DisplaySettings $pn
  grid $m.s -row 0 -column 0 -sticky w -pady {1.5m 0} -padx 2m
  grid $m.f -row 1 -column 0 -sticky ews
  grid $m.f2 -row 0 -column 1 -rowspan 2 -sticky s
  grid $m.log -row 2 -column 0 -columnspan 2 -sticky news -pady 2m -padx 2m
  grid $m.but -row 3 -column 0 -columnspan 2 -sticky ew -pady 2m -padx 2m
  grid rowconfigure $m 2 -weight 1
  grid columnconfigure $m 0 -weight 1

  HideShowOptions $pn

  bind $m <Alt-Key> [list ::tpad::AltKeyInDialog $m %A]
  bind $m <Key-Escape> [list ::tk::ButtonInvoke $m.but.close]

  ::tpad::PlaceWindow $pn $m
}
