## tpad version 1.3
# WinXP (TM) enhanced Notepad clone written in Tcl/Tk.
# Expat xmlwf plugin
# Copyright (C) 2004  Antonio Bonifati
#
# Please see the main file tpad.tcl for licence and disclaimer
#
## last revised on: $Date: 2004/08/31 $

package provide tpad 1.3

# make sure the msgcat namespace exists
namespace eval ::msgcat {}

namespace eval ::tpad::xmlwf {
  namespace export display

  namespace import ::msgcat::mc ::msgcat::mcmax
}

proc ::tpad::xmlwf::InitSettings {pn} {
  namespace eval $pn {}

  foreach var {external parameter standalone namespace} {
    set value [set ::tpad::xmlwf::$var]

    variable $pn\::$var $value
  }
}

proc ::tpad::xmlwf::EnableCheckbutton {pn} {
  variable ::tpad::$pn\::p
  variable $pn\::external

  set m $p.xmlwf
  set state [expr {$external ? {normal} : {disabled}}]
  $m.f.par configure -state $state
}

proc ::tpad::xmlwf::ProcessText {pn} {
  variable ::tpad::$pn\::p
  variable ::tpad::$pn\::filename
  foreach v {external parameter standalone namespace} {
    variable $pn\::$v
  }

  # generate the most simple command

  set cmd [list $::tpad::xmlwf]

  set opts {}
  if {$standalone} {append opts s}
  if {$namespace} {append opts n}
  if {$external} {
    if {$parameter} {
      append opts p
    } else {
      append opts x
    }
  }
  if {$opts!={}} {lappend cmd -$opts}

  # execute it and capture output

  # temporarily change to the same directory as the file
  set oldwd [pwd]
  cd [file dirname $filename]

  set xmlwfOut [eval exec $cmd << {[$p.text get 1.0 {end - 1 char}]}]

  # return to the previous directory
  cd $oldwd

  set out [split $xmlwfOut \n]
  if {$xmlwfOut!={}} {
    set last [lindex $out end]

    # parse last line, replacing STDIN with the current filename
    # and changing column numbers so that start from 1, not 0
    scan $last {STDIN:%d:%d: %n} row col msgIdx
    set msg [string range $last $msgIdx end]
    set displayfn [expr {$filename=={} ? [mc Untitled] : $filename}]
    lset out end "[file tail $displayfn]:$row:[expr $col + 1]: $msg"

    # set the cursor where the error occurred
    ::tpad::goToLine $pn $row $col
  }

  # update the log

  set w $p.xmlwf.log.txt

  $w configure -state normal
  $w delete 1.0 end
  $w insert 1.0 [join $out \n]
  $w configure -state disabled
}

proc ::tpad::xmlwf::display {pn} {
  variable ::tpad::$pn\::p

  if {![namespace exists $pn]} {
    InitSettings $pn
  }

  set m $p.xmlwf
  toplevel $m
  wm group $m $pn
  wm title $m [mc "XML well-formness checker"]
  wm transient $m $pn
  wm resizable $m no no
  # catch the kill of the window manager
  wm protocol $m WM_DELETE_WINDOW [namespace code "
    set ::tpad::$pn\::wf no
    destroy $m
  "]

  set w $m.f
  labelframe $w -padx 1.5m -pady 1.5m
  ::tpad::AmpWidget checkbutton $w.ext -text [mc "parse e&xternal entities"] \
    -var ::tpad::xmlwf::$pn\::external -onv yes -offv no \
    -comm [namespace code "EnableCheckbutton $pn"]
  $w configure -labelwidget $w.ext
  ::tpad::AmpWidget checkbutton $w.par -var ::tpad::xmlwf::$pn\::parameter \
    -text [mc "process &parameter entities"] -onv yes -offv no
  pack $w.par

  set w $m.f2
  frame $w -padx 1.5m -pady 1.5m
  ::tpad::AmpWidget checkbutton $w.stand -onv yes -offv no \
    -text [mc "error if not &standalone"] -var ::tpad::xmlwf::$pn\::standalone
  ::tpad::AmpWidget checkbutton $w.ns -text [mc "describe &namespaces"] \
    -var ::tpad::xmlwf::$pn\::namespace -onv yes -offv no
  pack $w.stand $w.ns -anchor w

  set w $m.but
  frame $w
  ::tpad::AmpWidget button $w.check -text [mc &Check] \
    -comm [namespace code "ProcessText $pn"]
  ::tpad::AmpWidget button $w.restore -text [mc "&Restore options"] -comm [
    namespace code "InitSettings $pn; EnableCheckbutton $pn"
  ]
  ::tpad::AmpWidget button $w.close -text [mc Cl&ose] -comm [namespace code "
    set ::tpad::$pn\::wf no
    focus $p.text
    after idle {destroy $m}
  "]
  pack $w.check $w.restore $w.close -side left -exp yes

  set w $m.log
  frame $w
  bind [::tpad::AmpWidget label $w.lab -text [mc "&Last Error Message"]] \
    <<AltUnderlined>> [list ::tk::TabToWindow $w.txt]
  text $w.txt -width 40 -height 2 -relief raised -setgrid true -state disabled \
    -font $::tpad::xmlwf::font -wrap none -xscroll "$w.scroll set"
  scrollbar $w.scroll -orient horizontal -comm "$w.txt xview"
  pack $w.lab $w.txt $w.scroll -fill x

  EnableCheckbutton $pn
  grid $m.f -row 0 -column 0 -pady 2m -padx 2m
  grid $m.f2 -row 0 -column 1 -pady 2m -padx 2m
  grid $m.log -row 1 -column 0 -columnspan 2 -sticky ew -pady 2m -padx 2m
  grid $m.but -row 2 -column 0 -columnspan 2 -sticky ew -pady 2m -padx 2m

  bind $m <Alt-Key> [list ::tpad::AltKeyInDialog $m %A]
  bind $m <Key-Escape> [list ::tk::ButtonInvoke $m.but.close]

  ::tpad::PlaceWindow $pn $m
}
